    {%- if metadata.description -%}
    <p>{{ metadata.description|safe }}</p>
    {%- endif -%}

    {%- if metadata.parameters -%}
    <div>
        <h2 style="margin-bottom: 0.2em;">Parameters</h2>
        {%- for param_name, param_metadata in metadata.parameters.items() if param_metadata.optional == false  -%}
        <span style="float:left">
            {{ param_name }}*
            {%- if 'type' in param_metadata -%}
            : <i>{{ param_metadata.type }}</i>
            {%- endif -%}
        </span>
        <span style="float: right">(*required)</span>
        <br/>
        {%- if 'description' in param_metadata -%}
        <p style='margin-left:2em'>
            {{ param_metadata.description }}
        </p>
        {%- endif -%}
        {%- endfor -%}

        {%- for param_name, param_metadata in metadata.parameters.items() if (param_metadata.optional == true) or (param_metadata.optional is undefined) -%}
        <span style="float: left">
            {{ param_name }}
            {%- if 'type' in param_metadata -%}
            : <i>{{ param_metadata.type }}</i>
            {%- endif -%}
        </span>
        {%- if 'default' in param_metadata -%}
        <span style="float: right">(default: {{ param_metadata.default }})</span>
        {%- endif -%}
        <br/>
        {%- if 'description' in param_metadata -%}
        <p style='margin-left:2em'>
            {{ param_metadata.description }}
        </p>
        {%- endif -%}
        {%- endfor -%}

    </div>
    {%- endif -%}

    {%- if metadata.inputs -%}
    <div>
        <h2 style="margin-bottom: 0.2em;">Inputs</h2>
        {% for input in metadata.inputs %}
        <span style="float:left">
            {%- if input.conditional -%}
            [{{ input.name }}]
            {%- else -%}
            {{ input.name }}
            {%- endif -%}
            {%- if 'type' in input -%}
            : <i>{{ input.type }}</i>
            {%- endif -%}
        </span>
        <br/>
        {%- if input.description -%}
        <p style='margin-left:2em'>
            {{ input.description }}
        </p>
        {%- endif -%}
        {%- endfor -%}
    </div>
    {%- endif -%}

    {%- if metadata.outputs -%}
    <div>
        <h2 style="margin-bottom: 0.2em;">Outputs</h2>
        {% for output in metadata.outputs %}
        <span style="float:left">
            {%- if output.conditional -%}
            [{{ output.name }}]
            {%- else -%}
            {{ output.name }}
            {%- endif -%}
            {%- if 'type' in output -%}
            : <i>{{ output.type }}</i>
            {%- endif -%}
        </span><br/>
        {%- if output.description -%}
        <p style='margin-left:2em'>
            {{ output.description }}
        </p>
        {%- endif -%}
        {%- endfor -%}
    </div>
    {%- endif -%}

    {%- if metadata.references -%}
    <p><strong>References</strong><br>
        {% for reference in metadata.references %}
        <span>[{{ loop.index }}]</span>
        <span style="margin-left: 1em">
            {%- if reference is string -%}
            {{ reference }}
            {%- else -%}
            <a href="{{ reference.url }}">{{ reference.text }}</a>
            {%- endif -%}
        </span><br>
        {% endfor %}
    </p>
    {%- endif -%}
